#    Kye - classic puzzle game
#    Copyright (C) 2005, 2006, 2007 Colin Phipps <cph@moria.org.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from os.path import basename
from random import Random
from string import upper
import gobject

from kye.common import tryopen, kyepaths
from kye.game import KGame
from kye.input import KyeRecordedInput, KDemoFormatError, KDemoFileMismatch

class KyeApp:
    def __init__(self, defaults, playfile="intro.kye", playlevel=""):
        self.__playfile = playfile
        self.__playlevel = playlevel
        self.__gamestate = "starting level"
        
        self.__recto = None
        self.__playback = None
        self.__game = None
        self.__defaults = defaults
        
    def run(self, frame):
        self.__frame = frame
        
        # Run first tick - loads the level - immediately
        self.do_tick()
        gobject.timeout_add(100,self.do_tick)

        self.__frame.main()
        
        # End any recording going on at the time of exit.
        self.__frame.moveinput.end_record()

    def do_tick(self):
        if self.__gamestate == "starting level":
            # Clean up any previous recording/playback title & close existing record
            self.__frame.moveinput.end_record()
            self.__frame.extra_title(None)
            
            # If recording this game, open the file to record to & tell the input system about it
            rng = Random()
            try:
                if self.__recto:
                    self.__frame.moveinput.record_to(self.__recto, playfile = self.__playfile, playlevel = self.__playlevel, rng = rng)
            except IOError, e:
                self.__frame.error_message(message="Failed to write to "+self.__recto)

            self.__recto = None
            if self.__frame.moveinput.is_recording():
                self.__frame.extra_title("Recording")

            # If playing a demo, open it & read the header
            self.__frame.moveinput.clear()
            ms = self.__frame.moveinput
            if self.__playback != None:
                try:
                    ms = KyeRecordedInput(self.__playfile, self.__playback)
                    self.__playlevel = ms.get_level()
                    ms.set_rng(rng)
                    self.__frame.extra_title("Replay")
                except KDemoFileMismatch, e:
                    self.__frame.error_message(message="Recording is for "+e.filename+"; you must load this level set first")
                except KDemoFormatError, e:
                    self.__frame.error_message(message="This file is not a Kye recording")
                except IOError, e:
                    self.__frame.error_message(message="Failed to read "+playback)
                self.__playback = None
            
            # Now try loading the actual level
            try:
                gamefile = tryopen(self.__playfile, kyepaths)
                self.__game = KGame(gamefile,lev = self.__playlevel, movesource = ms, rng = rng)

                # Add current level name to window title
                self.__frame.level_title(self.__game.thislev)
                # And remember that we have reached this level
                self.__defaults.add_known(self.__playfile,self.__game.thislev)
                # Put hint in the status bar
                self.__frame.stbar.update(hint=self.__game.hint,levelnum=self.__game.levelnum)
            except KeyError, e:
                self.__frame.error_message(message="Level "+self.__playlevel+" not known")
            except IOError, e:
                self.__frame.error_message(message="Failed to read "+self.__playfile)
            if self.__game != None:
                self.__gamestate = "playing level"
            else:
                self.__gamestate = ""

        if self.__gamestate == "playing level":
            # Check if the level has been completed.
            if self.__game.diamonds == 0:
                self.__gamestate="between levels"
                msg = self.__game.exitmsg
                if self.__frame.moveinput.is_recording():
                    msg = "Recording complete."
                if self.__frame.moveinput != self.__game.ms:
                    msg = "Playback complete."
                self.__frame.endleveldialog(self.__game.nextlevel, msg)
            
            # If we are still playing, run a gametick and update the screen.
            if self.__gamestate == "playing level":
                self.__game.dotick()
                self.__frame.canvas.game_redraw(self.__game,self.__game.invalidate)
                self.__frame.stbar.update(diamonds=self.__game.diamonds)
                if self.__game.thekye != None:
                    self.__frame.stbar.update(kyes=self.__game.thekye.lives)

        return True

    def restart(self, recordto = None, demo = None):
        self.__gamestate = "starting level"
        self.__recto = recordto
        self.__playback = demo
        self.__playlevel = self.__game.thislev

    def goto(self, lname):
        self.__gamestate = "starting level"
        self.__playlevel = upper(lname)

    def open(self, fname):
        self.__playfile = fname
        self.__playlevel = ""
        self.__gamestate = "starting level"

    def known_levels(self):
        return self.__defaults.get_known_levels(basename(self.__playfile))

